### First case study ##############################################################################

  ### Date: 30 March 2017

  ### Description:
  ###
  ### Bootstrap resampling for the 1st case study. Code is mostly identical to
  ### main analysis file, but replicated 1000 times



### Load packages #################################################################################

  library(foreach)
  library(doParallel)
  library(Matrix)
  library(mstate)
  


### Load and edit data ############################################################################

  data(prothr)
  dataset <- prothr
  dataset$length <- dataset$Tstop - dataset$Tstart
  ids.sample <- unique(dataset$id)


  
### Initiate multicore ############################################################################
  
  cluster <- makeCluster(16)
  registerDoParallel(cluster)
  
  results <- foreach(sample_no=1:1000,.packages=c("Matrix"),.verbose=TRUE) %dopar% {
  
    # Resample
    which.sample <- sample(ids.sample,rep=T)
    
    # Resampled dataset
    prothr <- dataset[dataset$id%in%c(which.sample),]
    
    # Transitions from normal (Placebo)
    normal.exposure <- sum(prothr$length[prothr$from==1&prothr$treat=="Placebo"]) 
    normal.transitions <- c(
      0,
      sum(prothr$from==1&prothr$to==2&prothr$status==1&prothr$treat=="Placebo"),
      sum(prothr$from==1&prothr$to==3&prothr$status==1&prothr$treat=="Placebo")
    )
    
    # Transitions from low (Placebo)
    low.exposure <- sum(prothr$length[prothr$from==2&prothr$treat=="Placebo"]) 
    low.transitions <- c(
      sum(prothr$from==2&prothr$to==1&prothr$status==1&prothr$treat=="Placebo"),
      0,
      sum(prothr$from==2&prothr$to==3&prothr$status==1&prothr$treat=="Placebo")
    )
    
    # 'Transitions' from death 
    death <- c(0,0,0)
    
    # Matrix of transition rates (Placebo)
    Tm.placebo <- rbind(normal.transitions/normal.exposure,
                        low.transitions/low.exposure,
                        death)
    
    # Transform into probabilities (Placebo)
    M <- diag(rowSums(Tm.placebo-diag(diag(Tm.placebo)))) +  - (Tm.placebo-diag(diag(Tm.placebo)))
    m <- dim(M)[1]
    P.placebo <- (diag(1,m)-1/2*M)%*%(solve(diag(1,m)+1/2*M ))
    rownames(P.placebo) <- c("Normal","Low","Dead")
    colnames(P.placebo) <- rownames(P.placebo)
    
    # Transitions from normal (Prednisone)
    normal.exposure <- sum(prothr$length[prothr$from==1&prothr$treat=="Prednisone"]) 
    normal.transitions <- c(
      0,
      sum(prothr$from==1&prothr$to==2&prothr$status==1&prothr$treat=="Prednisone"),
      sum(prothr$from==1&prothr$to==3&prothr$status==1&prothr$treat=="Prednisone")
    )
    
    # Transitions from low (Prednisone)
    low.exposure <- sum(prothr$length[prothr$from==2&prothr$treat=="Prednisone"]) 
    low.transitions <- c(
      sum(prothr$from==2&prothr$to==1&prothr$status==1&prothr$treat=="Prednisone"),
      0,
      sum(prothr$from==2&prothr$to==3&prothr$status==1&prothr$treat=="Prednisone")
    )
    
    # Matrix of transition rates (Prednisone)
    Tm.treatment <- rbind(normal.transitions/normal.exposure,
                          low.transitions/low.exposure,
                          death)
    
    # Transform into probabilities (Prednisone)
    M <- diag(rowSums(Tm.treatment-diag(diag(Tm.treatment)))) +  - (Tm.treatment-diag(diag(Tm.treatment)))
    m <- dim(M)[1]
    P.treatment <- (diag(1,m)-1/2*M)%*%(solve(diag(1,m)+1/2*M ))  
    
    rownames(P.treatment) <- rownames(P.placebo)
    colnames(P.treatment) <- rownames(P.placebo)
    
    # Build matrices for 10 years
    days <- 365*10
    
    m <- dim(P.treatment)[1]
    n <- sum(rowSums(P.treatment-diag(diag(P.treatment)))>0)
    
    U.treatment <- P.treatment[1:n,1:n]
    U.placebo <- P.placebo[1:n,1:n]
    
    death.treatment <- P.treatment[1:n,(n+1):m]
    death.placebo <- P.placebo[1:n,(n+1):m]
    
    tmp.treatment <- matrix(data=0,nrow=days*n+(m-n),ncol=days*n+(m-n))
    tmp.placebo <- matrix(data=0,nrow=days*n+(m-n),ncol=days*n+(m-n))
    
    rownames(tmp.treatment) <- c(rep(c("Normal","Low"),days),"Death")
    colnames(tmp.treatment) <- rownames(tmp.treatment)
    colnames(tmp.placebo) <- colnames(tmp.treatment)
    rownames(tmp.placebo) <- rownames(tmp.treatment)
    
    for(i in 1:(days-1)) {
      tmp.treatment[((i-1)*n+1):((i-1)*n+n),(i*n+1):(i*n+n)] <- U.treatment
      tmp.treatment[((i-1)*n+1):((i-1)*n+n),((days*n)+1):(days*n+(m-n))] <- death.treatment
    }
    tmp.treatment[(days*n-(n-1)):(days*n),(days*n-(n-1)):(days*n)] <- 0
    tmp.treatment[(days*n-(n-1)):(days*n),"Death"] <- 1
    tmp.treatment["Death","Death"] <- 1
    
    for(i in 1:(days-1)) {
      tmp.placebo[((i-1)*n+1):((i-1)*n+n),(i*n+1):(i*n+n)] <- U.placebo
      tmp.placebo[((i-1)*n+1):((i-1)*n+n),((days*n)+1):(days*n+(m-n))] <- death.placebo
    }
    tmp.placebo[(days*n-(n-1)):(days*n),(days*n-(n-1)):(days*n)] <- 0
    tmp.placebo[(days*n-(n-1)):(days*n),"Death"] <- 1
    tmp.placebo["Death","Death"] <- 1
    
    Pm.treatment <- Matrix(tmp.treatment)
    Pm.placebo <- Matrix(tmp.placebo)
    
    # Time spend in state
    
    m <- dim(Pm.treatment)[1]
    n <- sum(rowSums(Pm.treatment-diag(diag(Pm.treatment)))>0)
    
    N.placebo <- solve(diag(1,n)-Pm.placebo[1:n,1:n])
    N.treatment <- solve(diag(1,n)-Pm.treatment[1:n,1:n])
    
    n.placebo.normal <- rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")])
    n.placebo.low <-rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Low")])
    
    n.treatment.normal <- rowSums(as.matrix(N.treatment)[1:2,which(colnames(Pm.treatment)=="Normal")])
    n.treatment.low <- rowSums(as.matrix(N.treatment)[1:2,which(colnames(Pm.treatment)=="Low")])
    

    # Episodes spend in state
    
    R <- matrix(data=0,nrow=m,ncol=m)
    R[ which(rownames(Pm.treatment)=="Low"),which(colnames(Pm.treatment)%in%c("Normal"))] <- 1
    R <- Matrix(R)
    
    Z <- cbind(diag(1,n),matrix(data=0,ncol=m-n,nrow=n))
    Z <- Matrix(Z)
    
    e.treatment <- (N.treatment%*%Z%*%(Pm.treatment*R)%*%rep(1,m))[1:2]
    e.placebo <- (N.placebo%*%Z%*%(Pm.placebo*R)%*%rep(1,m))[1:2]
    
    # Get results
    c(n.placebo.normal,n.placebo.low,
      n.treatment.normal,n.treatment.low,
      e.treatment,e.placebo)
    
  }



### Save ##########################################################################################
  
  save(results,file="U:/Tmp/markov_spells_prednisone_bootstrap.rda")
