### Second case study ##############################################################################

  ### Date: 30 March 2017

  ### Description: 
  ###
  ### Code for calculating the number of episodes of disability in frail and
  ### nonfrail individuals. The data come from elderly study participants from
  ### New Haven, CT, USA, and were originally analyzed by Hardy et al. [1].
  ###
  ### References:
  ### [1] S. E. Hardy, J. A. Dubin, T. R. Holford, and T. M. Gill. Transitions between
  ###     states of disability and independence among older persons. American Journal of
  ###     Epidemiology, 161(6):575-584, 2005.



### Enter data ####################################################################################

  # Transition rates nonfrail: Transitions/Exposure
  nodis   <- c(  0,306, 58, 24)/ 22323
  milddis <- c(273,  0, 34, 10)/720
  sevdis  <- c( 23, 37,  0, 26)/178
  death   <- c(  0,  0,  0,  0)

  # Combine to matrix
  Tm <- rbind(nodis,milddis,sevdis,death)
  M <- diag(rowSums(Tm-diag(diag(Tm)))) +  - (Tm-diag(diag(Tm)))
  m <- dim(M)[1]
  P.nonfrail <- (diag(1,m)-1/2*M)%*%(solve(diag(1,m)+1/2*M ))

  # Transition rates frail: Transitions/Exposure
  nodis   <- c(  0,720,146, 22)/12491
  milddis <- c(661,  0,266, 23)/3412
  sevdis  <- c( 54,253,  0, 75)/1409
  death   <- c(  0,  0,  0,  0)

  # Combine to matrix
  Tm <- rbind(nodis,milddis,sevdis,death)
  M <- diag(rowSums(Tm-diag(diag(Tm)))) + -(t(Tm)-diag(diag(t(Tm))))
  m <- dim(M)[1]
  P.frail <- t(solve(diag(1,m)+1/2*M )%*%(diag(1,m)-1/2*M))
  
  
  
### Build transition matrices for 5 years #########################################################

  # Number of months; can be changed to, e.g., 120, to have 10 years instead of 5
  months <- 60  
  
  # Number of (transient) states
  m <- dim(P.frail)[1]
  n <- sum(rowSums(P.frail-diag(diag(P.frail)))>0)
  
  # Matrix U
  U.nonfrail <- P.nonfrail[1:n,1:n]
  U.frail <- P.frail[1:n,1:n]
  
  # Survival probabilities
  death.nonfrail <- P.nonfrail[1:n,(n+1):m]
  death.frail <- P.frail[1:n,(n+1):m]
  
  # Big empty matrices, filled below
  tmp.nonfrail <- matrix(data=0,nrow=months*n+(m-n),ncol=months*n+(m-n))
  tmp.frail <- matrix(data=0,nrow=months*n+(m-n),ncol=months*n+(m-n))
  rownames(tmp.nonfrail) <- c(rep(c("nodis","milddis","sevdis"),months),"death")
  colnames(tmp.nonfrail) <- rownames(tmp.nonfrail)
  colnames(tmp.frail) <- colnames(tmp.nonfrail)
  rownames(tmp.frail) <- rownames(tmp.nonfrail)
  
  # Fill matrix for every time step
  for(i in 1:(months-1)) {
    tmp.nonfrail[((i-1)*3+1):((i-1)*3+3),(i*3+1):(i*3+3)] <- U.nonfrail
    tmp.nonfrail[((i-1)*3+1):((i-1)*3+3),((months*n)+1):(months*n+(m-n))] <- death.nonfrail
  }
  # Set some transition probabilities
  tmp.nonfrail[(months*n-(n-1)):(months*n),(months*n-(n-1)):(months*n)] <- 0
  tmp.nonfrail[(months*n-(n-1)):(months*n),"death"] <- 1
  tmp.nonfrail["death","death"] <- 1
  
  # Fill matrix for every time step
  for(i in 1:(months-1)) {
    tmp.frail[((i-1)*3+1):((i-1)*3+3),(i*3+1):(i*3+3)] <- U.frail
    tmp.frail[((i-1)*3+1):((i-1)*3+3),((months*n)+1):(months*n+(m-n))] <- death.frail
  }
  # Set some transition probabilities
  tmp.frail[(months*n-(n-1)):(months*n),(months*n-(n-1)):(months*n)] <- 0
  tmp.frail[(months*n-(n-1)):(months*n),"death"] <- 1
  tmp.frail["death","death"] <- 1
  
  # Rename matrices
  Pm.nonfrail <- tmp.nonfrail
  Pm.frail <- tmp.frail

  
    
### Time spend in state ###########################################################################
  
  # Number of (transient) states
  m <- dim(Pm.frail)[1]
  n <- sum(rowSums(Pm.frail-diag(diag(Pm.frail)))>0)
  
  # Matrix N as per equation (1)
  N.frail <- solve(diag(1,n)-Pm.frail[1:n,1:n])
  N.nonfrail <- solve(diag(1,n)-Pm.nonfrail[1:n,1:n])
  
  # First column, upper part of Table 2
  rowSums(N.nonfrail[1:3,which(colnames(N.nonfrail)=="nodis")])[1]
  rowSums(N.nonfrail[1:3,which(colnames(N.nonfrail)=="milddis")])[1]
  rowSums(N.nonfrail[1:3,which(colnames(N.nonfrail)=="sevdis")])[1]
  
  # Second column, upper part of Table 2
  rowSums(N.frail[1:3,which(colnames(N.frail)=="nodis")])[1]
  rowSums(N.frail[1:3,which(colnames(N.frail)=="milddis")])[1]
  rowSums(N.frail[1:3,which(colnames(N.frail)=="sevdis")])[1]
  
  
  
### Episodes spend in state #######################################################################
  
  # Matrix of rewards as per equation (3)
  R <- matrix(data=0,nrow=m,ncol=m)
  R[ which(rownames(Pm.nonfrail)=="nodis"),which(colnames(Pm.nonfrail)%in%c("milddis","sevdis"))] <- 1
  
  # Matrix Z as needed for equation (2)
  Z <- cbind(diag(1,n),matrix(data=0,ncol=m-n,nrow=n))
  
  # Equation (2)
  (N.nonfrail%*%Z%*%(Pm.nonfrail*R)%*%rep(1,m))[1]
  (N.frail%*%Z%*%(Pm.frail*R)%*%rep(1,m))[1]

  # Average length of episodes as per equation (4)
  mdis <- rowSums(N.nonfrail[1:3,which(colnames(N.nonfrail)=="milddis")])
  sdis <- rowSums(N.nonfrail[1:3,which(colnames(N.nonfrail)=="sevdis")])
  denom <- (N.nonfrail%*%Z%*%(Pm.nonfrail*R)%*%rep(1,m))[1:3]
  (mdis+sdis)/denom
  
  mdis <- rowSums(N.frail[1:3,which(colnames(N.frail)=="milddis")])
  sdis <- rowSums(N.frail[1:3,which(colnames(N.frail)=="sevdis")])
  denom <- (N.frail%*%Z%*%(Pm.frail*R)%*%rep(1,m))[1:3]
  (mdis+sdis)/denom
 