### Load packages and data ########################################################################

  # Clear workspace
  rm(list=ls())
  
  # Packages
  library(mstate)
  library(Matrix)
  library(foreach)
  library(doParallel)
  
  # Data
  data(prothr)



### General settings ##############################################################################

  # Directory with files
  setwd("U:/Documents/__CODE__/Markov chain spells/V3/")
  
  # Directory for saving
  path.save <- "U:/Tmp/"
  
  # Seed
  set.seed(261)
  
  # Size of population (should be large relative to sample size)
  n.population <- 250000
  
  # Number of simulation replications
  n.replications <- 5000
  
  # Number of bootstrap replications per sample
  n.resamples <- 1000

  # Probability of censoring
  p.censoring <- 0.025
  


### Generate IID data #############################################################################      
  
  source("sim_generate_iid.r")
  source("sim_calculate_true.r")
  
  
  
### IID sims ######################################################################################
  
  n.sample <- 50
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_iid_bootstrap_",n.sample,".rda"))
  
  n.sample <- 100
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_iid_bootstrap_",n.sample,".rda"))
  
  n.sample <- 250
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_iid_bootstrap_",n.sample,".rda"))
  
  n.sample <- 500
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_iid_bootstrap_",n.sample,".rda"))
  
  n.sample <- 1000
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_iid_bootstrap_",n.sample,".rda"))
  
  
  
### Generate non-IID data #########################################################################      
  
  source("sim_generate_non_iid1.r")
  source("sim_calculate_true.r")  
  
  
  
### Non-IID sims ##################################################################################
  
  n.sample <- 50
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid1_bootstrap_",n.sample,".rda"))
  
  n.sample <- 100
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid1_bootstrap_",n.sample,".rda"))
  
  n.sample <- 250
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid1_bootstrap_",n.sample,".rda"))
  
  n.sample <- 500
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid1_bootstrap_",n.sample,".rda"))
  
  n.sample <- 1000
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid1_bootstrap_",n.sample,".rda"))
  
  
  
### Generate non-IID data 2 #######################################################################
  
  source("sim_generate_non_iid2.r")
  source("sim_calculate_true.r")  
  
  
  
### Non-IID sims ######################################################################################
  
  n.sample <- 50
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid2_bootstrap_",n.sample,".rda"))
  
  n.sample <- 100
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid2_bootstrap_",n.sample,".rda"))
  
  n.sample <- 250
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid2_bootstrap_",n.sample,".rda"))
  
  n.sample <- 500
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid2_bootstrap_",n.sample,".rda"))
  
  n.sample <- 1000
  source("sim_run.r")
  save(list=ls(),file=paste0(path.save,"MC_non_iid2_bootstrap_",n.sample,".rda"))
  
  
  
### END OF FILE ###################################################################################