### Sampling ######################################################################################

  cluster <- makeCluster(min(16,n.replications))
  registerDoParallel(cluster)
  
  # Cycle over replications
  results <- foreach(replication=1:n.replications,.packages=c("Matrix"),.verbose=TRUE) %dopar% {

    # Objects for bootstrap (reused for each replication)
    resampling.e.multinomial <- numeric(n.resamples)
    resampling.n.multinomial <- numeric(n.resamples)
    resampling.e.block <- numeric(n.resamples)
    resampling.n.block <- numeric(n.resamples)
    
    # Generate fake data
    which.dataset <- sample(ids.population,size=n.sample)
    dataset <- population[population$id%in%c(which.dataset),]
    ids.sample <- unique(dataset$id)
    ids.rows <- lapply(ids.sample,function(x) which(dataset$id %in% x) )
    names(ids.rows) <- paste(ids.sample)
    
    # Transition matrix of fake data
    firstrow <- tabulate(dataset$to[dataset$from==1])
    secondrow <- tabulate(dataset$to[dataset$from==2])
    thirdrow <- c(0,0,1)
    
    nfirstrow <- tabulate(dataset$from)[1]
    nsecondrow <- tabulate(dataset$from)[2]
    
    matchstates1 <- c(T,2:3%in%unique(dataset$to[dataset$from==1]))
    matchstates2 <- c(T,2:3%in%unique(dataset$to[dataset$from==2]))
    
    P1 <- numeric(3)
    P1[matchstates1] <- firstrow/nfirstrow
    P1 <- P1+c(0,0,0.001)
    P1 <- P1/sum(P1)
    
    P2 <- numeric(3)
    P2[matchstates2] <- secondrow/nsecondrow
    P2 <- P2+c(0,0,0.001)
    P2 <- P2/sum(P2)

    P.placebo <- rbind(P1,P2,P3)
    
    # Point estimates
    source("sim_sub_point.r",local=TRUE)

    # Save
    
    sample.e <- (N.placebo%*%Z%*%(Pm.placebo*R)%*%rep(1,m))[1]
    if(length(which(colnames(Pm.placebo)=="Normal"))==1) 
      sample.n <- as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")][1] else 
        sample.n <- rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")])[1]

    # Resampling sub-cycle
    for(numberofsample in 1:n.resamples) {

      # Multinomial resampling
      firstsample <- sample(1:3,size=nfirstrow,prob=P1,rep=T)
      secondsample <- sample(1:3,size=nsecondrow,prob=P2,rep=T)

      matchstates1 <- c(T,2:3%in%unique(firstsample))
      matchstates2 <- c(T,2:3%in%unique(secondsample))
      
      P1s <- numeric(3)
      P2s <- numeric(3)
      
      P1s[matchstates1] <- tabulate(firstsample)/nfirstrow
      P1s <- P1s+c(0,0,0.001)
      P1s <- P1s/sum(P1s)
      P2s[matchstates2] <- tabulate(secondsample)/nsecondrow
      P2s <- P2s+c(0,0,0.001)
      P2s <- P2s/sum(P2s)

      P.placebo <- rbind(P1s,P2s,P3)

      # Point estimates
      source("sim_sub_point.r",local=TRUE)

      # Save
      resampling.e.multinomial[numberofsample] <- (N.placebo%*%Z%*%(Pm.placebo*R)%*%rep(1,m))[1]
      if(length(which(colnames(Pm.placebo)=="Normal"))==1) 
        resampling.n.multinomial[numberofsample] <- as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")][1] else 
          resampling.n.multinomial[numberofsample] <- rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")])[1]

      # Block resampling
      which.sample <- sample(ids.sample,rep=T)
      block <- dataset[unlist(ids.rows[paste(which.sample)]),]

      firstrow <- tabulate(block$to[block$from==1])
      secondrow <- tabulate(block$to[block$from==2])
      
      nfirstrow <- tabulate(block$from)[1]
      nsecondrow <- tabulate(block$from)[2]
      
      matchstates1 <- c(T,2:3%in%unique(block$to[block$from==1]))
      matchstates2 <- c(T,2:3%in%unique(block$to[block$from==2]))
      
      P1s <- numeric(3)
      P2s <- numeric(3)
      
      P1s[matchstates1] <- firstrow/nfirstrow
      P1s <- P1s+c(0,0,0.001)
      P1s <- P1s/sum(P1s)
      
      P2s[matchstates2] <- secondrow/nsecondrow
      P2s <- P2s+c(0,0,0.001)
      P2s <- P2s/sum(P2s)

      P.placebo <- rbind(P1s,P2s,P3)

      # Point estimates
      source("sim_sub_point.r",local=TRUE)

      # Save
      resampling.e.block[numberofsample] <- (N.placebo%*%Z%*%(Pm.placebo*R)%*%rep(1,m))[1]
      if(length(which(colnames(Pm.placebo)=="Normal"))==1) 
        resampling.n.block[numberofsample] <- as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")][1] else 
          resampling.n.block[numberofsample] <- rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")])[1]

    }

    # Generate se estimates
    sample.e.block.se <- sd(resampling.e.block)*((n.resamples-1)/n.resamples)
    sample.n.block.se <- sd(resampling.n.block)*((n.resamples-1)/n.resamples)
    sample.e.multinomial.se <- sd(resampling.e.multinomial)*((n.resamples-1)/n.resamples)
    sample.n.multinomial.se <- sd(resampling.n.multinomial)*((n.resamples-1)/n.resamples)

    # Output results
    c(sample.e,sample.n,sample.e.block.se,sample.n.block.se,sample.e.multinomial.se,sample.n.multinomial.se)
    
  }
  
  stopCluster(cluster)

  
  
### Get results ###################################################################################
  
  # Get parallel results
  sample.e <- unlist(lapply(results,function(x) x[[1]]))
  sample.n <- unlist(lapply(results,function(x) x[[2]]))
  
  sample.e.block.se <- unlist(lapply(results,function(x) x[[3]]))
  sample.n.block.se <- unlist(lapply(results,function(x) x[[4]]))
   
  sample.e.multinomial.se <- unlist(lapply(results,function(x) x[[5]]))
  sample.n.multinomial.se <- unlist(lapply(results,function(x) x[[6]]))
  
  
  
### END OF FILE ###################################################################################  